@@ -217,21 +217,23 @@ class AbstractFigureAggregator(BaseFigure, ABC):
                 )
             else:
                 hf_series = self._to_hf_series(hf_trace_data["x"], hf_trace_data["y"])
-                start = hf_series.index[0] if start is None else start
-                end = hf_series.index[-1] if end is None else end
-                if hf_series.index.is_integer():
-                    start = round(start)
-                    end = round(end)
+                if len(hf_series) and (start is not None or end is not None):
+                    start = hf_series.index[0] if start is None else start
+                    end = hf_series.index[-1] if end is None else end
+                    if hf_series.index.is_integer():
+                        start = round(start)
+                        end = round(end)
 
-                # Search the index-positions
-                start_idx, end_idx = np.searchsorted(hf_series.index, [start, end])
-                hf_series = hf_series.iloc[start_idx:end_idx]
+                    # Search the index-positions
+                    start_idx, end_idx = np.searchsorted(hf_series.index, [start, end])
+                    hf_series = hf_series.iloc[start_idx:end_idx]
 
             # Return an invisible, single-point, trace when the sliced hf_series doesn't
             # contain any data in the current view
             if len(hf_series) == 0:
                 trace["x"] = [start]
                 trace["y"] = [None]
+                trace["text"] = ""
                 trace["hovertext"] = ""
                 return trace
 
@@ -266,17 +268,23 @@ class AbstractFigureAggregator(BaseFigure, ABC):
                     name = name[: -len(self._suffix)]
             trace["name"] = name
 
+            # Check if text also needs to be resampled
+            text = hf_trace_data.get("text")
+            if isinstance(text, (np.ndarray, pd.Series)):
+                # TODO -> extra logic is necessary for the detection and processing of
+                # non data-point selection downsamplers
+                trace["text"] = self._to_hf_series(x=hf_trace_data["x"], y=text).loc[
+                    s_res.index
+                ]
+            else:
+                trace["text"] = text
+
             # Check if hovertext also needs to be resampled
             hovertext = hf_trace_data.get("hovertext")
-            if isinstance(hovertext, pd.Series):
-                # TODO -> this can be optimized
-                trace["hovertext"] = pd.merge_asof(
-                    s_res,
-                    hovertext,
-                    left_index=True,
-                    right_index=True,
-                    direction="nearest",
-                )[hovertext.name].values
+            if isinstance(hovertext, (np.ndarray, pd.Series)):
+                trace["hovertext"] = self._to_hf_series(
+                    x=hf_trace_data["x"], y=hovertext
+                ).loc[s_res.index]
             else:
                 trace["hovertext"] = hovertext
             return trace
@@ -461,6 +469,7 @@ class AbstractFigureAggregator(BaseFigure, ABC):
                     'y': array([-0.01339909,  0.01390696,, ...,  0.25051913, 0.55876513]),
                     'axis_type': 'linear',
                     'downsampler': <plotly_resampler.aggregation.aggregators.LTTB at 0x7f786d5a9ca0>,
+                    'text': None,
                     'hovertext': None
                 },
             ]
@@ -471,6 +480,9 @@ class AbstractFigureAggregator(BaseFigure, ABC):
     def _to_hf_series(x: np.ndarray, y: np.ndarray) -> pd.Series:
         """Construct the hf-series.
 
+        By constructing the hf-series this way, users can dynamically adjust the hf
+        series argument.
+
         Parameters
         ----------
         x : np.ndarray
@@ -483,6 +495,15 @@ class AbstractFigureAggregator(BaseFigure, ABC):
         pd.Series
             The constructed hf_series
         """
+        # Note this is the same behavior as plotly support
+        # i.e. it also used the `values` property of the `x` and `y` parameters when
+        # these are pd.Series
+        if isinstance(x, pd.Series):
+            x = x.values
+
+        if isinstance(y, pd.Series):
+            y = y.values
+
         return pd.Series(
             data=y,
             index=x,
@@ -500,6 +521,7 @@ class AbstractFigureAggregator(BaseFigure, ABC):
         # Use these if you want some speedups (and are working with really large data)
         hf_x: Iterable = None,
         hf_y: Iterable = None,
+        hf_text: Union[str, Iterable] = None,
         hf_hovertext: Union[str, Iterable] = None,
         **trace_kwargs,
     ):
@@ -530,7 +552,9 @@ class AbstractFigureAggregator(BaseFigure, ABC):
         limit_to_view: boolean, optional
             If set to True the trace's datapoints will be cut to the corresponding
             front-end view, even if the total number of samples is lower than
-            ``max_n_samples``, By default False.
+            ``max_n_samples``, By default False.\n
+            Remark that setting this parameter to True ensures that low frequency traces
+            are added to the ``hf_data`` property.
         hf_x: Iterable, optional
             The original high frequency series positions, can be either a time-series or
             an increasing, numerical index. If set, this has priority over the trace its
@@ -538,9 +562,12 @@ class AbstractFigureAggregator(BaseFigure, ABC):
         hf_y: Iterable, optional
             The original high frequency values. If set, this has priority over the
             trace its data.
+        hf_text: Iterable, optional
+            The original high frequency text. If set, this has priority over the trace
+            its ``text`` argument.
         hf_hovertext: Iterable, optional
             The original high frequency hovertext. If set, this has priority over the
-            trace its ``text`` or ``hovertext`` argument.
+            trace its ```hovertext`` argument.
         **trace_kwargs: dict
             Additional trace related keyword arguments.
             e.g.: row=.., col=..., secondary_y=...
@@ -582,10 +609,10 @@ class AbstractFigureAggregator(BaseFigure, ABC):
         ---------
         * The ``NaN`` values in either ``hf_y`` or ``trace.y`` will be omitted! We do
           not allow ``NaN`` values in ``hf_x`` or ``trace.x``.
-        * ``hf_x``, ``hf_y``, and ``hf_hovertext`` are useful when you deal with large
-          amounts of data (as it can increase the speed of this add_trace() method with
-          ~30%). These arguments have priority over the trace's data and (hover)text
-          attributes.
+        * ``hf_x``, ``hf_y``, ``hf_text``, and ``hf_hovertext`` are useful when you deal
+          with large amounts of data (as it can increase the speed of this add_trace()
+          method with ~30%). These arguments have priority over the trace's data and
+          (hover)text attributes.
         * Low-frequency time-series data, i.e. traces that are not resampled, can hinder
           the the automatic-zooming (y-scaling) as these will not be stored in the
           back-end and thus not be scaled to the view.
@@ -602,27 +629,37 @@ class AbstractFigureAggregator(BaseFigure, ABC):
         trace.uid = uuid
 
         hf_x = (
-            trace["x"] if hasattr(trace, "x") and hf_x is None 
-            else hf_x.values if isinstance(hf_x, pd.Series)
-            else hf_x if isinstance(hf_x, pd.Index)
+            trace["x"]
+            if hasattr(trace, "x") and hf_x is None
+            else hf_x.values
+            if isinstance(hf_x, pd.Series)
+            else hf_x
+            if isinstance(hf_x, pd.Index)
             else np.asarray(hf_x)
         )
 
         hf_y = (
-            trace["y"] if hasattr(trace, "y") and hf_y is None 
-            else hf_y.values if isinstance(hf_y, (pd.Series, pd.Index))
+            trace["y"]
+            if hasattr(trace, "y") and hf_y is None
+            else hf_y.values
+            if isinstance(hf_y, (pd.Series, pd.Index))
             else hf_y
         )
         hf_y = np.asarray(hf_y)
 
-        # Note: "hovertext" takes precedence over "text"
+        hf_text = (
+            hf_text
+            if hf_text is not None
+            else trace["text"]
+            if hasattr(trace, "text") and trace["text"] is not None
+            else None
+        )
+
         hf_hovertext = (
             hf_hovertext
             if hf_hovertext is not None
             else trace["hovertext"]
             if hasattr(trace, "hovertext") and trace["hovertext"] is not None
-            else trace["text"]
-            if hasattr(trace, "text")
             else None
         )
 
@@ -645,12 +682,9 @@ class AbstractFigureAggregator(BaseFigure, ABC):
                 "(i.e., x and y, or hf_x and hf_y) to be <= 1 dimensional!"
             )
 
-            # Make sure to set the text-attribute to None as the default plotly behavior
-            # for these high-dimensional traces (scatters) is that text will be shown in
-            # hovertext and not in on-graph texts (as is the case with bar-charts)
-            trace["text"] = None
-
-            # Note: this also converts hf_hovertext to a np.ndarray
+            # Note: this also converts hf_text and hf_hovertext to a np.ndarray
+            if isinstance(hf_text, (list, np.ndarray, pd.Series)):
+                hf_text = np.asarray(hf_text)
             if isinstance(hf_hovertext, (list, np.ndarray, pd.Series)):
                 hf_hovertext = np.asarray(hf_hovertext)
 
@@ -661,6 +695,8 @@ class AbstractFigureAggregator(BaseFigure, ABC):
                 not_nan_mask = ~pd.isna(hf_y)
                 hf_x = hf_x[not_nan_mask]
                 hf_y = hf_y[not_nan_mask]
+                if isinstance(hf_text, np.ndarray):
+                    hf_text = hf_text[not_nan_mask]
                 if isinstance(hf_hovertext, np.ndarray):
                     hf_hovertext = hf_hovertext[not_nan_mask]
 
@@ -677,14 +713,6 @@ class AbstractFigureAggregator(BaseFigure, ABC):
 
             assert len(hf_x) == len(hf_y), "x and y have different length!"
 
-            # Convert the hovertext to a pd.Series if it's now a np.ndarray
-            # Note: The size of hovertext must be the same size as hf_x otherwise a
-            #   ValueError will be thrown
-            if isinstance(hf_hovertext, np.ndarray):
-                hf_hovertext = pd.Series(
-                    data=hf_hovertext, index=hf_x, copy=False, name="hovertext"
-                )
-
             n_samples = len(hf_x)
             # These traces will determine the autoscale RANGE!
             #   -> so also store when `limit_to_view` is set.
@@ -719,6 +747,7 @@ class AbstractFigureAggregator(BaseFigure, ABC):
                     "y": hf_y,
                     "axis_type": axis_type,
                     "downsampler": d,
+                    "text": hf_text,
                     "hovertext": hf_hovertext,
                 }
 
@@ -749,7 +778,8 @@ class AbstractFigureAggregator(BaseFigure, ABC):
                 self._print(f"[i] NOT resampling {trace['name']} - len={n_samples}")
                 trace.x = hf_x
                 trace.y = hf_y
-                trace.text = hf_hovertext
+                trace.text = hf_text
+                trace.hovertext = hf_hovertext
                 return super(self._figure_class, self).add_trace(
                     trace=trace, **trace_kwargs
                 )
@@ -763,8 +793,10 @@ class AbstractFigureAggregator(BaseFigure, ABC):
             if hasattr(trace, "y"):
                 trace["y"] = hf_y
 
-            if hasattr(trace, "text") and hasattr(trace, "hovertext"):
-                trace["text"] = None
+            if hasattr(trace, "text"):
+                trace["text"] = hf_text
+
+            if hasattr(trace, "hovertext"):
                 trace["hovertext"] = hf_hovertext
 
             return super(self._figure_class, self).add_trace(
@@ -873,14 +905,14 @@ class AbstractFigureAggregator(BaseFigure, ABC):
             elif len(autorange_matches) and not len(spike_matches):
                 # PreventUpdate returns a 204 status code response on the
                 # relayout post request
-                raise dash.exceptions.PreventUpdate()
+                return dash.no_update
 
         # If we do not have any traces to be updated, we will return an empty
         # request response
         if len(updated_trace_indices) == 0:
             # PreventUpdate returns a 204 status-code response on the relayout post
             # request
-            raise dash.exceptions.PreventUpdate()
+            return dash.no_update
 
         # -------------------- construct callback data --------------------------
         layout_traces_list: List[dict] = []  # the data
